#pragma once

#include "voice.h"
#include "AlgorithmicOscillator.h"
#include "MoogLadderFilter.h"

class CMiniSynthVoice : public CVoice
{
public:
	CMiniSynthVoice(void);
	~CMiniSynthVoice(void);

protected:
	// our synth components: Oscillators and Filters
	// Four oscillators
	CAlgorithmicOscillator m_Osc1;
	CAlgorithmicOscillator m_Osc2;
	CAlgorithmicOscillator m_Osc3;
	CAlgorithmicOscillator m_Osc4; // noise
	
	// 1 filter
	CMoogLadderFilter m_LPF;

	// hard sync is property of VOICE!
	double m_dHSRatio;

	// --- variables SPECIFIC to this voice architecture
	double m_dLFO1_OscModIntensity;
	double m_dLFO1_Filter1ModIntensity;
	double m_dLFO1_Filter2ModIntensity;
		
	double m_dLFO1_DCAAmpModIntensity;

	double m_dEG1_OscModIntensity;
	double m_dEG1_Filter1ModIntensity;
	double m_dEG1_Filter2ModIntensity;
	double m_dEG1_DCAAmpModIntensity;
	double m_dLFO1_DCAPanModIntensity;
	// ------------------------------------------------ //
	
	// waveshape modes
	enum {SawX3,SqrX3,SawSqrSaw,TriSawTri,TriSqrTri,HSSaw};

public:
	// intensity controls from UI -- these are all VOICE SPECIFIC! 
	inline void setOscEGIntensity(double dIntensity){m_dEG1_OscModIntensity = dIntensity;}
	inline void setDCAEGIntensity(double dIntensity){m_dEG1_DCAAmpModIntensity = dIntensity;}
	inline void setOscLFOIntensity(double dIntensity){m_dLFO1_OscModIntensity = dIntensity;}
	inline void setDCAPanLFOIntensity(double dIntensity){m_dLFO1_DCAPanModIntensity = dIntensity;}
	inline void setFilter1EGIntensity(double dIntensity){m_dEG1_Filter1ModIntensity = dIntensity;}
	inline void setFilter2EGIntensity(double dIntensity){m_dEG1_Filter2ModIntensity = dIntensity;}
	inline void setFilter1LFOIntensity(double dIntensity){m_dLFO1_Filter1ModIntensity = dIntensity;}
	inline void setFilter2LFOIntensity(double dIntensity){m_dLFO1_Filter2ModIntensity = dIntensity;}
	inline void setLFO1DCAAmpModIntensity(double dIntensity){m_dLFO1_DCAAmpModIntensity = dIntensity;}
	inline void setHSRatio(double d){m_dHSRatio = d;}
	
	inline virtual void setDetuneCents(double d)
	{
		// 3 ocsillators, detune two of them leave center pitch
		m_Osc1.m_nCents = d;
		m_Osc2.m_nCents = -d;
		
		// leave m_Osc3 alone
	}
	// EXAMPLE of how to modify the Modulation Matrix by adding or removing
	//         a SOURCE->DEST pairs
	//
	// FILTER KEY TRACK:
	//         SOURCE_MIDI_NOTE_NUM -> DEST_ALL_FILTER_KEYTRACK
	//
	inline void enableFilterKeyTrack(bool bEnable)
	{
		if(bEnable)
		{
			m_LPF.m_uSourceFcControl = DEST_FILTER1_KEYTRACK;

			// NOTE NUMBER -> FILTER Fc CONTROL
			modMatrixRow* pRow = new modMatrixRow;
			pRow->uSourceIndex = SOURCE_MIDI_NOTE_NUM;
			pRow->uDestinationIndex = DEST_ALL_FILTER_KEYTRACK;
			pRow->pModIntensity = &m_dFilterKeyTrackIntensity; // 0.5 = one octave below fc, 1 = fc, 2 = one octave above fc
			pRow->pModRange = &m_dDefaultModRange;
			pRow->uSourceTransform = TRANSFORM_NOTE_NUMBER_TO_FREQUENCY; 

			m_ModulationMatrix.addModMatrixRow(pRow);
		}
		else
		{
			m_LPF.m_uSourceFcControl = DEST_NONE;
			m_ModulationMatrix.removeMatrixRow(SOURCE_MIDI_NOTE_NUM, DEST_ALL_FILTER_KEYTRACK);
		}	
	}

	// VELOCITY TO ATTACK SCALING:
	//         SOURCE_VELOCITY -> DEST_ALL_EG_ATTACK_SCALING
	//
	inline void enableVelocityToAttackScaling(bool bEnable)
	{
		if(bEnable)
		{
			m_EG1.m_uModSourceEGAttackScaling = DEST_EG1_ATTACK_SCALING;

			// VELOCITY -> EG ATTACK SCALING
			modMatrixRow* pRow = new modMatrixRow;
			pRow->uSourceIndex = SOURCE_VELOCITY;
			pRow->uDestinationIndex = DEST_ALL_EG_ATTACK_SCALING;
			pRow->pModIntensity = &m_dDefaultModIntensity; // 1.0 for this synth
			pRow->pModRange = &m_dDefaultModRange;
			// 0 velocity -> scalar = 1, normal attack time
			// 128 velocity -> scalar = 0, fastest (0) attack time
			pRow->uSourceTransform = TRANSFORM_INVERT_MIDI_NORMALIZE; 

			m_ModulationMatrix.addModMatrixRow(pRow);
		}
		else
		{
			m_EG1.m_uModSourceEGAttackScaling = DEST_NONE;
			m_ModulationMatrix.removeMatrixRow(SOURCE_VELOCITY, DEST_ALL_EG_ATTACK_SCALING);
		}	
	}

	// MIDI NOTE # TO DECAY SCALING:
	//         SOURCE_MIDI_NOTE_NUM -> DEST_ALL_EG_DECAY_SCALING
	//
	inline void enableMIDINoteNumberToDecayScaling(bool bEnable)
	{
		if(bEnable)
		{
			m_EG1.m_uModSourceEGDecayScaling = DEST_EG1_DECAY_SCALING;

			// NOTE NUMBER -> EG DECAY SCALING
			modMatrixRow* pRow = new modMatrixRow;
			pRow->uSourceIndex = SOURCE_MIDI_NOTE_NUM;
			pRow->uDestinationIndex = DEST_ALL_EG_DECAY_SCALING;
			pRow->pModIntensity = &m_dDefaultModIntensity; // 1.0
			pRow->pModRange = &m_dDefaultModRange;
			// note#0 -> scalar = 1, normal decay time
			// note#128 -> scalar = 0, fastest (0) decay time
			pRow->uSourceTransform = TRANSFORM_INVERT_MIDI_NORMALIZE; 

			m_ModulationMatrix.addModMatrixRow(pRow);
		}
		else
		{
			m_EG1.m_uModSourceEGDecayScaling = DEST_NONE;
			m_ModulationMatrix.removeMatrixRow(SOURCE_MIDI_NOTE_NUM, DEST_ALL_EG_DECAY_SCALING);
		}	
	}

	// --- Overrides
	virtual void prepareForPlay();
	virtual void setSampleRate(double dSampleRate);
	virtual void update();
	virtual void reset();

	// --- Overriding: stuff specific to THIS synth voice
	inline virtual void setVoiceMode(UINT u)
	{
		// ignore if no real change
		if(m_uVoiceMode == u)return;

		m_uVoiceMode = u;
		
		// reset
		m_Osc1.m_bMasterOsc = false;
		
		// reset pitch of slave
		m_Osc2.m_dOscFo = m_dOscPitch;
				
		// osc3 is sub osc
		m_Osc3.m_nOctave = -1.0;

		switch(m_uVoiceMode)
		{
			case SawX3:
				m_Osc1.m_uWaveform = SAW1;
				m_Osc2.m_uWaveform = SAW1;
				m_Osc3.m_uWaveform = SAW1;
				break;

			case SqrX3:
				m_Osc1.m_uWaveform = SQUARE;
				m_Osc2.m_uWaveform = SQUARE;
				m_Osc3.m_uWaveform = SQUARE;
				break;

			case SawSqrSaw:
				m_Osc1.m_uWaveform = SAW1;
				m_Osc2.m_uWaveform = SQUARE;
				m_Osc3.m_uWaveform = SAW1;
				break;
		
			case TriSawTri:
				m_Osc1.m_uWaveform = TRI;
				m_Osc2.m_uWaveform = SAW1;
				m_Osc3.m_uWaveform = TRI;
				break;

			case TriSqrTri:
				m_Osc1.m_uWaveform = TRI;
				m_Osc2.m_uWaveform = SQUARE;
				m_Osc3.m_uWaveform = TRI;
				break;
		
			case HSSaw:
				m_Osc1.m_uWaveform = SAW1;
				m_Osc2.m_uWaveform = SAW1;
							
				// NOTE: 2 is master; Osc1 ALWAYS runs at the MIDI Note Pitch
				m_Osc1.m_bMasterOsc = true;
				
				m_Osc3.m_uWaveform = SAW1; // this is ignored anyway
				break;

			default:
				m_Osc1.m_uWaveform = SAW1;
				m_Osc2.m_uWaveform = SAW1;
				m_Osc3.m_uWaveform = SAW1;
				break;
		}

		m_Osc1.reset();
		m_Osc2.reset();
		m_Osc3.reset();
	}

	inline virtual bool doVoice(double& dLeftOutput, double& dRightOutput)
	{
		// this does basic on/off work zero impact on speed
		if(!CVoice::doVoice(dLeftOutput, dRightOutput))
			return false;

		// --- ARTICULATION BLOCK --- //
		// layer 1 modulators
		m_EG1.doEnvelope();
		m_EG2.doEnvelope();	
		m_EG3.doEnvelope();
		m_EG4.doEnvelope();

		// update and do LFO	
		m_LFO1.updateOscillator();	
		m_LFO1.doOscillate();
	
		// apply modulatora
		m_ModulationMatrix.doModulationMatrix();
		
		// update for attack/decay mods
		m_EG1.updateEG();
		m_EG2.updateEG();
		m_EG3.updateEG();
		m_EG4.updateEG();

		// update DCA and Filter
		m_DCA.updateDCA();	
		m_LPF.updateFilter();

		// slave is osc2 (running at higher freq)
		if(m_uVoiceMode == HSSaw)
		{
			double dRatioMod = m_dHSRatio*(m_ModulationMatrix.m_dDestinations[m_uSourceFoRatio]);
			m_Osc1.m_dOscFo = m_dOscPitch;
			m_Osc2.m_dOscFo = m_dOscPitch*dRatioMod;
		}

		// update oscillators
		m_Osc1.updateOscillator();
		m_Osc2.updateOscillator();
		m_Osc3.updateOscillator();

		// --- DIGITAL AUDIO ENGINE BLOCK --- //
		double dOscMix = 0.0;
		if(m_uVoiceMode == HSSaw)
			dOscMix = m_Osc1.doOscillate();
		else
			dOscMix = 0.333*m_Osc1.doOscillate() + 
					  0.333*m_Osc2.doOscillate() +
					  0.333*m_Osc3.doOscillate() +
					  m_Osc4.doOscillate();

		// apply the filter
		double dLPFOut = m_LPF.doFilter(dOscMix);

		// apply the DCA
		m_DCA.doMonoDCA(dLPFOut, dLeftOutput, dRightOutput);

		return true;
	}
};
